"""
DOT labeller for OpenMP.
"""

from ctree.dotgen import DotGenLabeller

# ---------------------------------------------------------------------------
# DOT generator

class OmpDotLabeller(DotGenLabeller):
    """
    Visitor to generator DOT.
    """
    pass
