"""
DOT labeller for Cilk.
"""

from ctree.dotgen import DotGenLabeller

# ---------------------------------------------------------------------------
# DOT labeller

class CilkDotLabeller(DotGenLabeller):
    """
    Visitor to label DOT nodes.
    """
    pass
